"""
Path     addons/source-python/packages/custom/k2tools/auth/consumer.py
Name     Authentificaion Consumer
Version  1.00.000
Revision $Rev: 27 $
Author   (C) [#OMEGA] - K2


INFO

<Info>


AGREEMENT

See k2tools_license.txt
"""

# =============================================================================
# >> IMPORTS
# =============================================================================
# Python Imports

# =====================
# Source Python Imports
from events.manager import EventRegistry as _EventRegistry

# =====================
# Auth imports
from .shared import Flag
from .shared import Permission
from .shared import Group
from .shared import Player
from .shared import PlayerCached
from .shared import flags
from .shared import permissions
from .shared import groups
from .provider import _auth_providers

# =============================================================================
# >> INTERNAL (PRIVATE) CLASSES
# =============================================================================

class _Cache(dict):
    def __init__(self):
        pass #TODO add every player  - though obsolete when the server starts?
        # There are no players on init.. however, this may get called
        # at some other time but server start?
    def _rebuild(self):
        for userid in self:
            self[userid] = AuthCached(userid)

# =============================================================================
# >> EXTERNAL (PUBLIC) CLASSES
# =============================================================================

# Normal usage
class Auth(object):
    """
    Real-time authenification
    """
    def __init__(self, userid):
        # TODO Check for valid userid
        
        self.player_objects = [] 
        
        global _auth_providers
        for auth_provider in _auth_providers:
            for auth_type in auth_provider.auth_types:
                obj = auth_type['func'](userid)
                if obj:
                    self.player_objects.append(obj)
                
    def is_authed(self): 
        """
        Checks whether the user is authenficated by any auth provider.
        
        @return True if the user is authentificated though at least 1 auth
            provider.
        @return False otherwise
        """
        return bool(self.player_objects)
    def get_immunity(self):
        """
        @return int the immunity level
        """
        immunity = 0
        for obj in self.player_objects:
            if obj.immunity > immunity:
                immunity = obj.immunity
        return immunity
    def has_flag(self, flag):
        f = flags[flag]
        for obj in self.player_objects:
            # Deny is checked first
            if f in obj.dflags:
                return False
            for group in obj.groups:
                if f in groups.dflags:
                    return False
            # If everything is fine, proceed
            if f in obj.flags:
                return True
            for group in obj.groups:
                if f in group.flags:
                    return True
        return False
    def in_group(self, group):
        g = groups[group]
        for obj in self.player_objects:
            if g in obj.groups:
                return True
        return False
    def has_permission(self, perm):
        p = permissions[perm]
        for obj in self.player_objects:
            # User may have the permission denied
            if p in obj.dpermissions:
                return True
            # Flag may have the permission denied
            for flag in obj.dflags:
                # Both must be checked since as soon the flag is denied
                # all the permissions are also denied. In addition
                # the flag may have denied permissions anyway
                if p in flag.permissions + flag.dpermissions:
                    return False
            # A normal flag may have the permission denied
            for flag in obj.dflags:
                if p in flag.dpermissions:
                    return False
            for group in obj.groups:
                # Group may have the permission denied
                if p in group.dpermissions:
                    return False
                # A flag assigned to that group may have the permission
                for flag in group.dflags:
                    # Both must be checked. See few lines above.
                    if p in flag.permissions + flag.dpermissions:
                        return False
                # In addition, an normal flag may have the permission
                # denied
                for flag in group.flags:
                    if perm in flag.dpermissions:
                        return False
            
            # User may have the permission
            if p in obj.permissions:
                return True
            # Flag may have the permission
            for flag in obj.flags:
                if p in flag.permissions:
                    return True
            for group in obj.groups:
                # Group may have the permission
                if p in group.permissions:
                    return True
                # A flag assigned to that group may have the permission
                for flag in group.flags:
                    if p in flag.permissions:
                        return True
        return False

    
class AuthCached(Auth):
    """
    Creates a cache on creation
    """
    def __init__(self, userid):
        super(AuthCached, self).__init__(userid)
        self.cached_object = PlayerCached()
        for obj in self.player_objects:
            # Denied Permissions
            for perm in obj.dpermissions:
                self._add_item(self.cached_object.dpermissions, perm)
            # Denied Flags
            for flag in obj.dflags:
                self._add_item(self.cached_object.dflags, flag)
                # Careful: a denied's flag normal permissions 
                # and denied permissions are checked
                for perm in flag.permissions + flag.dpermissions:
                    self._add_item(self.cached_object.dpermissions, perm)
            # A normal flag may have a denied permission
            for flag in obj.flags:
                for perm in flag.dpermissions:
                    self._add_item(self.cached_object.dpermissions, perm)
            # Denied flags / permissions that are inherited by a group
            for group in obj.groups:
                if group not in self.cached_object.groups:
                    self.cached_object.groups.append(group)
                    # Add permissions to cache
                    for perm in group.dpermissions:
                        self._add_item(self.cached_object.dpermissions, perm)
                    # Check for denied flags
                    for flag in group.dflags:
                        self._add_item(self.cached_object.dflags, flag)
                        # Also inherited to cache
                        for perm in flag.permissions + flag.dpermissions:
                            if perm not in self.cached_object.dpermissions:
                                self.cached_object.dpermissions.append(perm)
                    # Check for normal flags with denied permissions
                    for flag in group.flags:
                        for perm in flag.dpermissions:
                            self._add_item(self.cached_object.dpermissions, perm)
                        
            # Add permissions to cache
            for perm in obj.permissions:
                self._add_item(self.cached_object.permissions, perm)
            # Add flags to cache
            for flag in obj.flags:
                self._add_item(self.cached_object.flags, flag)
                # Also inherited to cache
                for perm in flag.permissions:
                    self._add_item(self.cached_object.permissions, perm)
            for group in obj.groups:
                if group not in self.cached_object.groups:
                    self.cached_object.groups.append(group)
                    # Add permissions to cache
                    for perm in group.permissions:
                        self._add_item(self.cached_object.permissions, perm)
                    # Add flags to cache
                    for flag in group.flags:
                        self._add_item(self.cached_object.flags, flag)
                        # Also inherited to cache
                        for perm in flag.permissions:
                            self._add_item(self.cached_object.permissions, perm)
        # Wow, that was a lot of work :P Now let's build a proper cache
        # that removes denied permissions/flags
        self.cached_object.cflags = self.cached_object.flags
        for flag in self.cached_object.flags:
            if flag in self.cached_object.dflags:
                self.cached_object.cflags.remove(flag)
        self.cached_object.cpermissions = self.cached_object.permissions
        for permission in self.cached_object.permissions:
            if permission in self.cached_object.dpermissions:
                self.cached_object.cpermissions.remove(permission)
    def _add_item(self, obj, item):
        if item not in obj:
            obj.append(item)
    def has_flag(self, flag):
        return flag in self.cached_object.flags
    def in_group(self, group):
        return group in self.cached_object.groups
    def has_permission(self, perm):
        return perm in self.cached_object.permissions
        

# =============================================================================
# >> PRIAVTE (INTERNAL) FUNCTIONS
# =============================================================================

# Event listeners
def _player_connect(event):
    userid = event.get_int('userid')
    _cache[userid] = AuthCached(userid)
def _player_disconnect(event):
    del _cache[event.get_int('userid')]
def _player_authentificated(event):
    pass # TODO, when steamid is authenficiated
    '''
    userid = #TODO get userid
    for auth_provider in _auth_providers:
        for auth_type in auth_provider.auth_types:
            # refreshing on this event is only requried if any of the
            # auth providers actually does steamid authentification.
            if auth_type['type'] == 'steamid':
                _cache[userid] = AuthCached(userid)
                # Once it's refreshed, we don't need to loop any further
                return
    '''
#

def _refresh_cache():
    global _cache
    for item in _cache.keys():
        _cache[item] = AuthCached(item)

# =============================================================================
# >> PUBLIC (EXTERNAL) FUNCTIONS
# =============================================================================

def get_cached_object(userid):
    """
    @userid - Userid of the player you want to get the cached object from
    
    @return AuthCached object from the cache for the specified player.
    """
    return _cache[userid]
        
def register_permission(name, flags=[], groups=[]):
    """
    registers a new permission to use.
    
    @name   - Name of the permission to register
    @flags  - list of default flags for this permission [Optional]
    @groups - list of default groups for this permission [Optional]
    
    @raise KeyError if the permission exists
    """
    if name in permissions:
        raise KeyError("Permission %s already exists" % name)
    p = Permission(name)
    
    for flag in flags:
        f = flags[flag]
        # Add a reference to the flag so we know what flag this permission uses
        p.flags.append(f)
        # Add a reference to the permission so we can retrive the permissions
        # for a specific flag easily
        f.permissions.append(p)
    
    for group in groups:
        g = groups[group]
        # Add a reference to the group so we know what groups this permission
        # uses
        p.groups.append(g)
        # Add a reference to the permission so we can retrive the permissions
        # for a specific group easily
        g.permissions.append(p)
        
    permissions[name] = p

def unregister_permission(name):
    """
    Unregisters a permission.
    
    @raise KeyError if the permission does not exist
    """
    p = permissions[name]
    
    # Remove the references to this permission for every flag
    for flag in p.flags:
        flag.permissions.remove(p)
    
    # Remove the references to this permission for every group
    for group in p.groups:
        group.permissions.remove(p)
    
    del permissions[name]
    
# =============================================================================
# >> INITIALIZE / STARTUP STUFF
# =============================================================================

_cache = _Cache()
_EventRegistry.register_for_event('player_connect', _player_connect)
_EventRegistry.register_for_event('player_disconnect', _player_disconnect)
#TODO _EventRegistry.register_for_event('player_authentificated', _player_authentificated)