"""
Path     addons/source-python/packages/custom/k2tools/auth/provider.py
Name     Authentificaion Provider
Version  1.00.000
Revision $Rev: 27 $
Author   (C) [#OMEGA] - K2


INFO

<Info>


AGREEMENT

See k2tools_license.txt
"""

# =============================================================================
# >> EXTERNAL (PUBLIC) CLASSES
# =============================================================================

class AuthProvider(object):
    """
    The authprovider class.
    
    You must subclass this class in your own auth provider and override
    the methods.
    """
    def __init__(self):
        """
        AuthProvider Constructor
        
        This should take of any intialization that you need to have done.
        When subclassing you may also want to call the original
        constructor to take care of setting up some of the required data.
        """
    def _add_auth_method(self, method, authtype=None):
        """
        Use this function in your constructor (__init__) to add new 
        authentification methods. Using this method ensures that your 
        authprovider still works even after updates to the API have been 
        made.
        
        @method   - the method that will be called when authentification
                    is requestetd.
                    It must accept one parameter which is the userid
        @authtype - Method of using auth. As of now, only 'steamid' is
                    used in a special case for cached access.
                    I suggest using stuff like:
                    steamid, ip, name
        """
        if not hasattr(self, 'auth_types'):
            self.auth_types = []
        self.auth_types.append({'func': method, 'type': authtype})
    def refresh(self):
        """
        Refreshes the auth priviledges
        """
        raise NotImplementedError()
        
# =============================================================================
# >> FUNCTIONS
# =============================================================================
        
def register(obj):
    """
    Registers an auth provider.
    
    @obj - Authprovider object instance
    """
    if not isinstance(obj, AuthProvider):
        raise TypeError("The authprovider MUST subclass AuthProvider")
    _auth_providers[obj] = {}

def unregister(obj):
    """
    Unregisters an auth provider.
    
    @obj - Authprovider object instance
    """
    if not isinstance(obj, AuthProvider):
        raise TypeError("The authprovider MUST subclass AuthProvider")
    del _auth_providers[obj]
    
# =============================================================================
# >> INITIALIZE / STARTUP STUFF
# =============================================================================

_auth_providers = {}