"""
Path     addons/source-python/packages/custom/k2tools/auth/shared.py
Name     Authentificaion Provider
Version  1.00.000
Revision $Rev: 27 $
Author   (C) [#OMEGA] - K2


INFO

<Info>


AGREEMENT

See k2tools_license.txt
"""

# =============================================================================
# >> INTERNAL (PRIVATE) CLASSES
# =============================================================================

class _Flags(dict):
    def __missing__(self, key):
        f = Flag(key)
        self[key] = f
        
        # By default this flag is also added to the special groups
        groups['restricted'].dflags.append(f)
        groups['root'].flags.append(f)
        
        return f
        
class _Groups(dict):
    def __missing__(self, key):
        self[key] = Group(key)
        return self[key]

class _Permissions(dict):
    def __missing__(self, key):
        p = Permission(key)
        self[key] = p
        
        # This permission is new, so it get's added automatically to the
        # special groups and flags
        flags['restricted'].dpermissions.append(p)
        groups['restricted'].dpermissions.append(p)
        flags['root'].permissions.append(p)
        groups['root'].permissions.append(p)
        
        return p

# =============================================================================
# >> EXTERNAL (PUBLIC) CLASSES
# =============================================================================

class Permission(object):
    """
    The Permission Class.
    
    The purpose of this class is to present an authenficication object for
    a single permission. It does not include any inherited information, only
    information specific to the permission.
    
    Attributes:
    name   - Name of the permission
    flags  - List of flag objects
    dflags - List of denied flag objects
    groups - List of groups
    """
    def __init__(self, name):
        self.name = name
        self.flags = []
        self.dflags = []
        self.groups = []
    def __eq__(self, other):
        if not isinstance(other, Permission):
            return False
        if other.name != self.name:
            return False
        if other.flags != self.flags:
            return False
        if other.dflags != self.dflags:
            return False
        if other.groups != self.groups:
            return False
        return True


class Flag(object):
    """
    The Flag Class.
    
    The purpose of this class is to present an authenficication object for
    a single flag. It does not include any inherited information, only
    information specific to the flag.
    
    Attributes:
    name         - Name of the flags
    permissions  - List of permissions
    dpermissions - List of denied permissions
    """
    def __init__(self, name):
        self.name = name
        self.permissions = []
        self.dpermissions = []

class Group(object):
    """
    The Group Class.
    
    The purpsoe of this class is to present an authenficication object for
    a single group. It does not include any inherited information, only
    information specific to the group.
    
    Attributes:
    name         - Name of the permission
    flags        - List of flag objects
    dflags       - List of denied flag objects
    permissions  - List of permissions
    dpermissions - List of denied permissions
    immunity     - Immunity level granted by this group
    """
    def __init__(self, name):
        self.name = name
        self.flags = []
        self.dflags = []
        self.permissions = []
        self.dpermissions = []
        self.immunity = 100

class Player(object):
    """
    The Player Class.
    
    The purpsoe of this class is to present an authenficication object for
    a single player. It does not include any inherited information, only
    information specific to the player.
    
    Attributes:
    flags        - List of flag objects
    dflags       - List of denied flag objects
    permissions  - List of permissions
    dpermissions - List of denied permissions
    groups       - List of groups
    immunity     - Immunity level
    """
    def __init__(self):        
        self.flags = []
        self.dflags = []
        self.permissions = []
        self.dpermissions = []
        self.groups = []
        self.immunity = 100
       

class PlayerCached(Player):
    """
    The PlayerCached Class. 
    
    This class inherits from Player, as such, all the Player attributes are 
    available. In addition, the following may be used:
    
    Attributes:
    cflags       - List of combined flags (denied flags removed)
    cpermissions - List of combined permissions (denied permissions removed)
    """
    def __init__(self):
        super(PlayerCached, self).__init__()
        # Combined flags & permissions
        self.cflags = []
        self.cpermissions = []
        
# =============================================================================
# >> INITIALIZE / STARTUP STUFF
# =============================================================================

flags = _Flags()
permissions = _Permissions()
groups = _Groups()

