"""
Path     addons/source-python/packages/custom/k2tools/langlib.py
Name     Translation Library
Version  1.00.000
Revision $Rev: 36 $
Author   (C) [#OMEGA] - K2


INFO

Based on the ES langlib to maintain compatibility


AGREEMENT

See k2tools_license.txt

Otherwise it should be noted that the design of the files was made by the
Eventscripts team. The original langlib is freely available under the MIT 
license.
"""

__all__ = ['Strings']
__version__ = '1.00.000 $Rev: 36 $'
__author__ = '[#OMEGA] - K2'
__date__ = '$Date: 2013-09-30 15:48:45 +0200 (Mo, 30 Sep 2013) $'

# =============================================================================
# >> Imports
# =============================================================================

# Python
from logging import getLogger
from os.path import join
from string import Template

# Site-Packages
from configobj import ConfigObj

# Source.Python
from paths import SP_DATA_PATH

# =============================================================================
# >> Globals
# =============================================================================

_logger = getLogger('k2tools.langlib')

# =============================================================================
# >> Methods
# =============================================================================


def _get_languages():
    """
    Parses addons/source-python/_libs/languages.ini for language identification
    """
    global _languages
    _languages = ConfigObj(
        SP_DATA_PATH.joinpath('languages.ini'), encoding='utf-8')
    
    # Cached list for performance reasons (faster then looping every time)
    global _languages_ids
    _languages_ids = []
    for item in _languages:
        _languages_ids.append(_languages[item])

# =============================================================================
# >> Classes
# =============================================================================


class Strings(ConfigObj):
    def __init__(self, *args, **kwargs):
        """
        Creates a new translations Strings object from a file with ConfigObj.
        You can pass any arguments to ConfigObj
        
        @args    - positional arguments to pass
        @kwargs - keyword arguments to pass
        """
        self._logger = _logger
        if 'encoding' not in kwargs:
            kwargs['encoding'] = 'utf-8'
        super(Strings, self).__init__(*args, **kwargs)
    
    def __call__(self, text, opts={}, lang='en'):
        """
        Looks up the translation for a given text id for the given language and
        replaces any keywords with the values given in opts. 
        If the translation is not found, the message ID will be shown.
        If the formatting fails, the unformatted message will be shown.
        
        @text - text id  for the lookup
        @opts - dictionary for replacing any $keywords 
        @lang - language id or string

        @return str translated and format string
        """
        return self._expand(text, opts, self._get_language_abbreviation(lang))
    
    def _get_language_abbreviation(self, lang):
        """
        Tries to get the appropriate language abbreviation for the specified 
        language.
        
        @return str id if the language was found
        @return str 'en' if the language was not found.
        """
        if lang in _languages:
            return _languages[lang]
        elif lang in _languages_ids:
            return lang
        else:
            self._logger.warn('Unrecognized language "%s"', lang)
            return 'en'
    
    def _expand(self, text, opts, lang):
        if text not in self:
            self._logger.warn('Translation with ID "%s" not found', text)
            return text

        if lang not in self[text]:
            if lang == 'en':
                return text
            return self(text, opts, 'en')

        try:
            return Template(self[text][lang]).substitute(opts)
        except KeyError:
            self._logger.warn((
                'Argument subtituion failed for translation with ID "%s" and '
                'language "%s"'), text, lang)
            return self[text][lang]

# =============================================================================
# >> Init
# =============================================================================

_get_languages()
