"""
Path     addons/source-python/packages/custom/k2tools/misc.py
Name     Miscellaneous Functions and Classes
Version  1.00.000
Revision $Rev: 39 $
Author   (C) [#OMEGA] - K2


INFO

Miscellaneous functions and classes that do not fit into other categories.
Generally all methods here can be subject to change or be moved into specific
python modules.


AGREEMENT

See k2tools_license.txt
"""

__all__ = ['console_message', 'get_team_color', 'get_player', 'get_maxplayers']
__version__ = '1.00.000 $Rev: 39 $'
__author__ = '[#OMEGA] - K2'
__date__ = '$Date: 2014-02-09 22:15:20 +0100 (So, 09 Feb 2014) $'

# =============================================================================
# >> Imports
# =============================================================================

# Source Python C++
from conversions_c import index_from_edict
from entity_c import EntityGenerator
from player_c import PlayerGenerator

# Source Python Python
from core import GameEngine

# k2tools
from ._shared import cfg as _cfg

# =============================================================================
# >> Methods
# =============================================================================


def console_message(message):
    """
    Sends a message to console. If logging is enabled, it will be logged, 
    otherwise it will just print out.
    
    @message - message to print
    """
    if GameEngine.is_log_enabled():
        GameEngine.log_print(message + '\n')
    else:
        print (message)


def get_team_color(teamid):
    """
    Returns a color code for the given teamid depending on the specific game. 
    
    @teamid - ID of the team
    
    @return str the color code
    """
    return _cfg['Colors']['Team%s' % teamid]


def get_player(string=''):
    """
    Finds the first CPlayerInfo instance for the specified search string.
    
    Resolution order:
    - UserID
    - SteamID
    - Exact Name
    - Partial Name
    
    @string - Search string
    
    @return CPlayerInfo on success
    @return None on failure
    """
    # Empty string; return first player
    if not string:
        for player in PlayerGenerator():
            return player
        return None
    
    if string.isdigit():
        id = int(string)
        # Matching userid
        for player in PlayerGenerator():
            if player.get_userid() == id:
                return player
    # Matching SteamID
    for player in PlayerGenerator():
        if player.get_networkid_string() == string:
            return player
    # Matching Name (exact)
    for player in PlayerGenerator():
        if player.get_name() == string:
            return player
    # Matching Name (partial)
    for player in PlayerGenerator():
        if string in player.get_name():
            return player
    # Failed to find a match, return None
    return None

    
def get_maxplayers():
    """
    Calculates current maxplayers and returns it.
    
    @return int maxplayers
    """
    index = 255
    for edict in EntityGenerator('cs_team_manager'):
        tempindex = index_from_edict(edict)
        if tempindex < index:
            index = tempindex
    return index - 1
