"""
Path     addons/source-python/plugins/k2tools_test_data/k2tools_test_keyvalues.py
Name     Keyvalues Test
Version  1.00.000
Revision $Rev: 31 $
Author   (C) [#OMEGA] - K2


INFO

Tests for keyvalues


AGREEMENT
 
See k2tools_license.txt
"""

# =============================================================================
# >> Imports
# =============================================================================

# Source.Python
from paths import GAME_PATH
from paths import PLUGIN_PATH

# K2Tools
import k2tools.keyvalues

#import imp;imp.reload(k2tools.keyvalues)

# =============================================================================
# >> Load & Unload
# =============================================================================

def load():
    kv = k2tools.keyvalues.KeyValuesFile()
    kv['test'] = 1
    kv['test2'] = 'test'
    kv['test3'] = 3.0
    kv['subkey'] = k2tools.keyvalues.KeyValues(kv)
    kv['subkey']['test'] = 'Subkey'
    
    kv['deleteme'] = 1
    del kv['deleteme']
    
    kv_copy = kv.copy()
    kv_deep_copy = kv.deep_copy()
    kv['subkey']['test'] = 'New'
    
    # For kv.walk
    control = [
        (kv, 'test', 1, 0),
        (kv, 'test2', 'test', 0),
        (kv, 'test3', 3.0, 0),
        (kv, 'subkey', kv['subkey'], 0),
        (kv['subkey'], 'test', 'New', 1),
    ]
    
    # For testing merge
    kv_merge = k2tools.keyvalues.KeyValues()
    kv_merge['test'] = 10
    kv_merge['test4'] = 1
    
    assert kv['test'] == 1, 'Added key is of wrong value'
    assert 'test' in kv, 'Added key is not in keyvalues'
    assert 'deleteme' not in kv, 'Deleted key is still in keyvalues' 
    assert kv.get_depth() == 0, 'Key of depth 0 has invalid length'
    assert kv['subkey'].get_depth() == 1, 'key of depth 1 has invalid ' \
        'length'
    assert ['test', 'test2', 'test3', 'subkey'] == kv.keys(), 'Key did not' \
        'match the test object'
    assert [1, 'test', 3.0, kv['subkey']] == kv.values(), 'Values did not ' \
        'match the test object'
    assert kv_copy == kv, 'The shallow copy should be equal to the original ' \
        'kv'
    assert kv_deep_copy != kv, 'The deep copy should not be equal to the ' \
        'orginal kv'
    assert kv.walk() == control, 'Walk should match the test object'
    
    # Merge tests
    kv.merge(kv_merge)
    assert kv['test'] == 10, 'Should be overriden to 10 after merge'
    assert kv['test4'] == 1, 'Should be a new key from merge'
    
    
    # Test KeyValuesFile methods
    p = PLUGIN_PATH.joinpath('k2tools_test_keyvalues', 'test.kv')
    kv.write(p)
    kv.load(p)
    
    assert kv_deep_copy != kv, 'KeyValues should be the same before and ' \
        'writing to disk'
    
    p = PLUGIN_PATH.joinpath('k2tools_test_keyvalues', 'test2.kv')
    kv.load(p, typecast=True)
    
    test2 = (
        ('test', 10),
        ('depth2', 2.4),
        ('depth4', 4),
        ('test4', 'a/b/c'),
    )
    
    id = 0
    for kvo, key, value, depth in kv.walk():
        if not isinstance(key, k2tools.keyvalues.KeyValues) and \
                not isinstance(value, k2tools.keyvalues.KeyValues):
            assert key == test2[id][0], 'Parsed key does not match test key'
            assert value == test2[id][1], 'Parsed value does not match ' \
                'test value'
            id += 1