"""
Path     addons/source-python/plugins/k2tools_test_popup/k2tools_test_popup.py
Name     Popup Tests
Version  1.00.000
Revision $Rev: 34 $
Author   (C) [#OMEGA] - K2


INFO

Tests for popup

AGREEMENT
 
See k2tools_license.txt
"""

# =============================================================================
# >> Imports
# =============================================================================

from translations.strings import LangStrings

from k2tools.cmdlib import AdvancedCommandManager
from k2tools.popup.constants import *
from k2tools.popup.popups import *
from k2tools.popup.options import *

# =============================================================================
# >> Load/Unload
# =============================================================================

def callback_translate(playerinfo, option, popup, language):
    return {'testcb': 'Callback Arguments work'}

def callback(playerinfo, option, popup):
    print (playerinfo)
    print (option)
    print (popup)

def prehook(playerinfo, option, popup, language):
    if option._name.startswith('allowed'):
        return ACCESS_ALLOWED
    elif option._name.startswith('restricted'):
        return ACCESS_RESTRICTED
    elif option._name.startswith('denied'):
        return ACCESS_DENIED
    return None

def show_menu(av):
    menu.send(av['playerinfo'])

# =============================================================================
# >> Load/Unload
# =============================================================================

def load():
    # Do tests on the list/dict style methods
    basic = Popup()
    basic.append('removeplz')
    basic.append('test2')
    basic.append('test3')
    index = basic.index('removeplz')
    basic.pop(index)
    basic.remove('test3')
    basic.insert(0, 'test1')
    assert ['test1', 'test2'] == [option._name for option in basic._options]
    del basic[0]
    del basic['test2']
    basic['newkey'] = 'newkey'
    
    error = False
    try:
        basic['error'] = 'lol'
    except KeyError:
        error = True
    assert error, 'Should raise KeyError if key does not match optio name'
    
    basic[0] = 'newopt'
    
    assert ['newopt'] == [option._name for option in basic._options]
    
    try:
        basic[1] = 'WTF'
    except IndexError:
        error = True
    assert error, 'Should raise IndexError if index is out of bounds'
    
    assert len(basic) == len(basic._options) == len(basic._options_str), 'Both internal objects must be the same size'
    
    basic.clear()
    
    assert len(basic) == 0 and len(basic._options) == 0 and len(basic._options_str) == 0, 'When cleared, the popup be empty'
    
    l = []
    lobj = []
    for i in range(0, 5):
        n = 'test%s' % i
        o = PopupOption(n)
        l.append(n)
        lobj.append(o)
        basic.append(o)
    
    assert l == basic.keys(), 'Keys must contain all the option names'
    assert lobj == basic.values(), 'Values must contain all the option objects'
    
    # Popup functionality tests
    basic_translate = Popup(translations=LangStrings('k2tools/k2tools_test_popup'))
    basic_translate.append('test1')
    basic_translate.append('test2')
    basic_translate.append('test3')
    
    error = False
    try:
        basic_translate.append('test4')
    except KeyError:
        error = True
    assert error, 'Should raise an KeyError if the option has a translation ID which doesn\'t exist'
    
    basic_translate.append(PopupOption('test4', value='test4_translation'))
    basic_translate.append('test_arg')
    basic_translate.append(PopupOption('test_arg_cb', translate_callback=callback_translate))
    
    error = False
    try: basic_translate.append('test4')
    except KeyError: error = True
    assert error
    
    list = PopupList()
    for i in range(0, 20):
        list.append('item%s' % i)
    
    test_title = Popup()
    test_title.set_title('Test Title')
    
    test_description = Popup()
    test_description.set_description('Test Description')
    
    global menu
    menu = PopupMenu()
    menu.append(PopupOption('basic',value='Basic Popup', callback=basic, prehook=prehook))
    menu.append(PopupOption('basic_translate',value='Basic + Translations Popup', callback=basic_translate, callback_kwargs={'targs': {'test': 'Arguments work!'}}, prehook=prehook))
    menu.append(PopupOption('list',value='List', callback=list, prehook=prehook))
    menu.append(PopupOption('inactive',value='Should be inactive', active=False, callback=callback, prehook=prehook))
    menu.append(PopupOption('allowed_test',value='Should be allowed', active=True, callback=callback, prehook=prehook))
    menu.append(PopupOption('restricted_test',value='Should be restricted', active=True, callback=callback, prehook=prehook))
    menu.append(PopupOption('denied_test',value='Should be denied', active=True, callback=callback, prehook=prehook))
    menu.append(PopupOption('no7',value='Should be option 1. and on a new page'))
    menu.append(PopupOption('test_title', active=True, callback=test_title))
    menu.append(PopupOption('test_description', active=True, callback=test_description))

    AdvancedCommandManager.register('test',
        callback=(show_menu,),
        log_pre_exec=True,
        log_post_exec=True,
        log_spam_protect=True,
        console=True,
    )

def unload():
    AdvancedCommandManager.unregister('test')