"""
Path     addons/source-python/k2tools_wikidoc/k2tools_wikidoc.py
Name     K2Tools Wikidoc Generator
Version  1.00.000
Revision $Rev: 34 $
Author   (C) [#OMEGA] - K2


INFO

Generate MediaWiki for k2tools pages from a source file.

Still some issues:
- unsupported stuff, like class/static methods, decorators, etc, (only need that when it's used in my libs though) 


AGREEMENT

See addons/source-python/_libs/k2tools/license.txt

"""

# =============================================================================
# >> Imports
# =============================================================================

# Python
import glob
import inspect
import imp
from collections import OrderedDict

# K2Tools
from k2tools._paths import K2TOOLS_PACKAGE_PATH

# =============================================================================
# >> Classes
# =============================================================================

class DocReturn(object):
    def __init__(self):
        self.doc = ''
        self.doclines = []
        self.args = {}
        self.exceptions = []
        self.returns = []

class GenerateWiki(object):
    def __init__(self, modulename):
        self.modulename = modulename
        self.module = __import__(modulename, fromlist=(modulename.split('.')[0]))
        imp.reload(self.module)
        self.data = {
            'classes': OrderedDict(),
            'exception_classes': OrderedDict(),
            'instances': OrderedDict(),
            'methods': OrderedDict(),
            'attributes': OrderedDict(),
        }
    def _docgen(self, docstr, remove_empty=False):
        docparse = DocReturn()
        if not docstr:
            return docparse
        last = ''
        multiline_type = None
        doclines = docstr.replace('\r','').split('\n')
        if not doclines[-1].strip():
            doclines = doclines[:-1]
        if not doclines[0].strip():
            doclines = doclines[1:]
        for line in doclines:
            line = line.strip()
            if remove_empty and not line:
                continue 
            if line.startswith('@'):
                if line.startswith('@return'):
                    last = ''
                    l = line[8:].split(maxsplit=1)
                    multiline_type = docparse.returns
                    docparse.returns.append({'type': l[0], 'description': l[1]})
                elif line.startswith('@raise'):
                    last = ''
                    l = line[7:].split(maxsplit=1)
                    multiline_type = docparse.exceptions
                    docparse.exceptions.append({'type': l[0], 'description': l[1]})
                else:
                    l = line[1:].split('-', 1)
                    last = l[0].strip()
                    docparse.args[last] = l[1]
            elif last:
                docparse.args[last] += '<br />\n' + line
            elif multiline_type:
                if line: # Multi-line comments with empty lines are unsupported
                    multiline_type[-1]['description'] += '<br />\n' + line
            else:
                docparse.doclines.append(line)
        docparse.doc = '<br />\n'.join(docparse.doclines)
        return docparse
    def _get_argspec_delta(self, argspec):
        if argspec.defaults:
            return len(argspec.args) - len(argspec.defaults)
        else:
            return len(argspec.args)
    def _get_class_def(self, cls):
        """Format a class like python does"""
        class_def = []
        class_def.append('(')
        # Every second entry seems to contain the inheritance list of our class
        # It is a list with a single entrry, so we can safely use [0]
        # Now it's a tuple containing (cls, subclasses)
        for subcls in inspect.getclasstree((cls, ))[1][0][1]:
            class_def.append('%s, ' % subcls.__name__)
        return ''.join(class_def).rstrip(', ') + ')'
            
    def _get_func_def(self, func_name, argspec):
        """Format the function like python does"""
        func_desc = []
        #func_desc.append(func_name)
        func_desc.append('(')
        if argspec.args:
            delta = self._get_argspec_delta(argspec)
            for arg in argspec.args[:delta]:
                func_desc.append('%s, ' % arg)
            for arg in argspec.args[delta:]:
                default = argspec.defaults[argspec.args.index(arg)-delta]
                # Make sure empty strings show up
                if isinstance(default, str):
                    default = '"%s"' % default
                func_desc.append('%s=%s, ' % (arg, default))
        if argspec.varargs:
            func_desc.append('*%s, ' % argspec.varargs)
        if argspec.kwonlyargs:
            for arg in argspec.kwonlyargs:
                func_desc.append('%s=%s, ' % (arg, argspec.kwonlydefaults[arg]))
        if argspec.varkw:
            func_desc.append('**%s, ' % argspec.varkw)
        return ''.join(func_desc).rstrip(', ') + ')'
    def _inspect_args(self, argspec, argdoc):
        args = OrderedDict()
        if 'self' in argspec.args:
            argspec.args.remove('self')
        if argspec.args:
            delta = self._get_argspec_delta(argspec)
            for arg in argspec.args:
                data = {'description':  '', 'default': ''}
                if arg in argdoc:
                    data['description'] = argdoc[arg]
                if argspec.defaults:
                    i = argspec.args.index(arg)
                    if i >= delta:
                        data['default'] = argspec.defaults[i - delta]
                       
                args[arg] = data
        if argspec.varargs:
            args[argspec.varargs] = {
                'description': argdoc[argspec.varargs] if argspec.varargs in argdoc else 'positional arguments',
                'default': '',
                }
        if argspec.kwonlyargs:
            for arg in argspec.kwonlyargs:
                args[arg] = {
                    'description': argdoc[arg],
                    'default': argspec.kwonlydefaults[arg],
                }
        if argspec.varkw:
            args[argspec.varkw] = {
                'description': argdoc[argspec.varkw] if argspec.varkw in argdoc else 'keyword arguments',
                'default': '',
            }
        # Add any args that haven't been added. Need this for kwargs that are implicit
        for arg in argdoc:
            if arg not in args:
                args[arg] = {
                    'description': argdoc[arg],
                    'default': '',
                }
        return args
                               
    def walk(self, obj, datadict, inclass=False, ininstance=False):
        d = dir(obj)
        # This is prettier later in the wiki
        if '__init__' in d:
            d.remove('__init__')
            d.insert(0, '__init__')
        for itemstr in d:
            if inclass:
                valid = ['__init__', '__call__', '__getitem__', '__setitem__', '__delitem__', '__contains__', '__iter__', '__next__', '__repr__', '__str__', '__eq__', '__ne__']
                if ininstance:
                    valid.remove('__init__')
            else:
                valid = []
            if itemstr not in valid and itemstr.startswith('_'): # might want to doc __init__
                continue
            if hasattr(obj, '__all__') and itemstr not in obj.__all__:
                continue
            # fromkeys is neither a function nor a method ???
            if itemstr in ('fromkeys', ):
                continue
            item = getattr(obj, itemstr)
            if inspect.isfunction(item):
                # Skip inherited methods
                if inspect.isclass(obj) and obj.__name__ not in item.__qualname__:
                    continue
                docparse = self._docgen(item.__doc__)
                func_name = item.__name__
                argspec = inspect.getfullargspec(item)
                
                datadict['methods'][func_name] = {
                    'fulldef': self._get_func_def(func_name, argspec),
                    'description': docparse.doc,
                    'arguments': self._inspect_args(argspec, docparse.args),
                    'inclass': int(inclass),
                    'returns': docparse.returns,
                    'exceptions': docparse.exceptions,
                }
            elif inspect.isclass(item):
                if issubclass(item, Exception):
                    key = 'exception_classes'
                else:
                    key = 'classes'
                datadict[key][item.__name__] = OrderedDict(
                    fulldef = self._get_class_def(item),
                    attributes = OrderedDict(),
                    methods = OrderedDict(),
                    properties = OrderedDict(),
                    description = self._docgen(item.__doc__).doc,
                    )
                self.walk(item, datadict[key][item.__name__], inclass=True)
            elif isinstance(item, property):
                datadict['properties'][itemstr] = {
                    'description': self._docgen(item.__doc__).doc,
                }
            elif hasattr(obj, '__k2doc__') and itemstr in obj.__k2doc__:
                if obj.__k2doc__[itemstr] == 'instance':
                    # Don't actually use the class name (i.e cls.__name__)
                    cls = item.__class__
                    datadict['instances'][itemstr] = OrderedDict(
                        fulldef = self._get_class_def(cls),
                        attributes = OrderedDict(),
                        methods = OrderedDict(),
                        properties = OrderedDict(),
                        description = self._docgen(cls.__doc__).doc,
                        )
                    self.walk(cls, datadict['instances'][itemstr], inclass=True, ininstance=True)
            else:
                # Skip inherited methods
                if inspect.isclass(obj) and obj.__name__ not in item.__qualname__:
                    continue
                docparse = self._docgen(obj.__doc__)
                if itemstr in docparse.args:
                    desc = docparse.args[itemstr]
                else:
                    desc = ''
                datadict['attributes'][itemstr] = {'description': desc}
    def doc_property(self, dict, level=' '):
        for item in dict:# TODO - broken
            self.outstr.append('{{Property\n')
            self.outstr.append('| name=%s\n' % itemstr)
            self.outstr.append('| description=%s\n' % self._docgen(item.__doc__).doc) 
            self.outstr.append('}}\n')
    def doc_class(self, dict, level=' ', instance=False, exception=False):
        for item in dict:
            self.outstr.append('%s{{Class\n' % (level, ))
            self.outstr.append('%s| name=%s\n' % (level, item))
            self.outstr.append('%s| fulldef=%s\n' % (level, dict[item]['fulldef']))
            self.outstr.append('%s| description=%s\n' % (level, dict[item]['description']))
            if instance:
                self.outstr.append('%s| type=instance\n' % (level, ))
            if exception:
                self.outstr.append('%s| type=exception\n' % (level, ))
            
            self.outstr.append('%s| methods=' % (level, ))
            if dict[item]['methods']:
                self.doc_func(dict[item]['methods'], level+' ')
            else:
                self.outstr.append('\n')
            
            self.outstr.append('%s| attributes=' % (level, ))
            if dict[item]['attributes']:
                self.doc_attr(dict[item]['attributes'], level+' ')
            else:
                self.outstr.append('\n')
            
            self.outstr.append('%s}}\n' % (level, ))
    def doc_func(self, dict, level=' '):
        for item in dict:
            self.outstr.append('%s{{Function\n' % (level, ))
            self.outstr.append('%s| name=%s\n' % (level, item))
            self.outstr.append('%s| fulldef=%s\n' % (level, dict[item]['fulldef']))
            self.outstr.append('%s| description=%s\n' % (level, dict[item]['description']))
            
            self.outstr.append('%s| returns=' % (level, ))
            if dict[item]['returns']:
                self.doc_return(dict[item]['returns'], level+' ')
            else:
                self.outstr.append('\n')
                
            self.outstr.append('%s| exceptions=' % (level, ))
            if dict[item]['exceptions']:
                self.doc_exception(dict[item]['exceptions'], level+' ')
            else:
                self.outstr.append('\n')
            
            self.outstr.append('%s| arguments=' % (level, ))
            if dict[item]['arguments']:
                self.doc_arg(dict[item]['arguments'], level+' ')
            else:
                self.outstr.append('\n')
            
            self.outstr.append('%s| method=%s}}\n' % (level, dict[item]['inclass']))
    def doc_exception(self, list, level=' '):
        for item in list:
            self.outstr.append('%s{{Exception\n' % (level, ))
            self.outstr.append('%s| type=%s\n' % (level, item['type']))
            self.outstr.append('%s| description=%s\n' % (level, item['description']))
            self.outstr.append('%s}}' % (level, )) # No new line or media wiki goes nuts :P     
    def doc_return(self, list, level=' '):
        for item in list:
            self.outstr.append('%s{{Return\n' % (level, ))
            self.outstr.append('%s| type=%s\n' % (level, item['type']))
            self.outstr.append('%s| description=%s\n' % (level, item['description']))
            self.outstr.append('%s}}' % (level, )) # No new line or media wiki goes nuts :P
    def doc_attr(self, dict, level=' '):
        for item in dict:
            self.outstr.append('%s{{Attribute\n' % (level, ))
            self.outstr.append('%s| name=%s\n' % (level, item))
            self.outstr.append('%s| description=%s}}\n' % (level, dict[item]['description']))
    def doc_arg(self, dict, level=' '):
        for item in dict:
            self.outstr.append('%s{{Argument\n' % (level, ))
            self.outstr.append('%s| name=%s\n' % (level, item))
            self.outstr.append('%s| default=%s\n' % (level, dict[item]['default']))
            self.outstr.append('%s| description=%s}}\n' % (level, dict[item]['description']))
    def generate(self):
        self.walk(self.module, self.data)
        self.outstr = []
        self.outstr.append('{{Module\n')
        # Remove unnecesary stuff like paths, rev, agreement
        lines = self._docgen(self.module.__doc__, remove_empty=False).doclines
        out = []
        write = False
        skip = False
        for line in lines:
            if skip:
                skip = False
                continue
            if line.startswith('INFO'):
                write = True
                skip = True
                continue
            elif line.startswith('AGREEMENT'):
                # Remove a line break
                out = out[:-1]
                break
            if write:
                out.append(line)
        self.outstr.append('| overview=%s' % '<br \>\n'.join(out))
        if self.data['classes']:
            self.outstr.append('| classes=')
            self.doc_class(self.data['classes'])
        if self.data['exception_classes']:
            self.outstr.append('| exceptions=')
            self.doc_class(self.data['exception_classes'], exception=True)
        if self.data['instances']:
            self.outstr.append('| instances=')
            self.doc_class(self.data['instances'], instance=True)
        if self.data['methods']:
            self.outstr.append('| functions=')
            self.doc_func(self.data['methods'])
        if self.data['attributes']:
            self.outstr.append('| attributes=')
            self.doc_attr(self.data['attributes'])
        self.outstr.append('}}\n')
        self.outstr.append('[[Category: K2Tools]]')
        return self.outstr
    
for path in glob.glob(K2TOOLS_PACKAGE_PATH + '/*.py'):
    mod = path.split('\\')[-1]
    if mod.startswith('_'):
        continue
    if mod in ('download_manager.py', ):
        continue
    mod = 'k2tools.' + mod.replace('.py','')
    with open('Y:/' + mod + '.txt','w') as f:
        f.writelines(GenerateWiki(mod).generate())
        
for path in glob.glob(K2TOOLS_PACKAGE_PATH + '/popup/*.py'):
    mod = path.split('\\')[-1]
    if mod.startswith('_'):
        continue
    if mod in ('download_manager.py', ):
        continue
    mod = 'k2tools.popup.' + mod.replace('.py','')
    with open('Y:/' + mod + '.txt','w') as f:
        f.writelines(GenerateWiki(mod).generate())